︠631c614d-22b4-4c2e-b4d9-b36723c65bd5︠
#1st order differential equations (o.d.e) for refraction:
#dr/ds = u, where u = cos(theta)
#du/ds = (1-u^2)*(1/r+1/n*dn/dr)
#n(r) = 1+10^-6*a*exp(-b*r), where n is index of refraction (Exp Model)
#dn/dr = 1+10^-6*a*-b*exp(-b*r)
︡2310e592-6cf5-4ef5-9a3a-6dc8e1a73021︡︡
︠ec8645aa-f578-4b7b-b0a9-7f995e7caae7︠
#Pick initial condition:
#theta = 30 degrees, in radians: pi/6
#u = cos(pi/6) = 0.86602
#r = 1.7(per m)*10^4
︡22fe65a8-7113-49a6-b22e-507890e0fb20︡︡
︠e5e858ad-be0b-4450-9e6e-ea0a94938996︠
radian = var('radian')
radian = pi/6
u = cos(radian).n(digits = 5)
u #Returns initial condition of u
︡7c688757-abda-41ee-9dc4-6d0dfbb53b34︡{"stdout": "0.86602"}︡
︠e055deb2-8ff0-4089-9873-c9b00fca97e4︠
#From problem 3, where r = h:
#a = 340
#b = 1.4090(per m)*10^4
︡171a185a-2f50-4aac-b4c2-8d9fde45e697︡︡
︠067907c0-30e2-448e-b584-5ff2bcce7dec︠
#Use 2nd Order Runge-Kutta to integrate equation du/ds = (1-u^2)*(1/r+1/n*dn/dr)
︡f5ba3ef7-f723-49c5-8835-b42fb72e350d︡︡
︠53b77605-d5a2-416c-978b-7ed84264f1d9︠
def udot(r,u):
    a = 340
    b = 1.4090
    return (1-(u)^2)*((1)/(r))+((1)/(1+10^-6*a*exp(-b*r)))*(1+10^-6*a*-b*exp(-b*r))
︡2a8c2bd1-7fa1-4ffd-abcb-ddb8d950b4e8︡︡
︠a926039d-0f6e-4664-a5b9-0915da2702f7︠
import scipy #Transfer files from scipy (open source library of algorithms and mathematical tools from Python programming language) into this worksheet.

def rk2(u0,r0,rn,rt): #2nd Order Runge-Kutta defined.

    radial_dis = scipy.arange(r0,rn,rt) #Creates array range (arange) of radial_dis values.
    u=[] #Stores u = dr/ds or derivative of radial distance with respect to (s) measurement along               
         #the ray path.
    for r in radial_dis:
        u.append(u0) #Attach results (u0) to variable u.
        k1=udot(r,u0)
        k2=udot(r+0.5*rt,u0+0.5*k1)
        u0=u0+(k1+k2)/2
    return radial_dis,u
︡492be9b0-3b87-442d-af6f-42b3c9899e2b︡︡
︠15c0fc7e-c9e3-49a3-898a-b5d2b71df7a0︠
rk2(0.86602, 1.71, 3, 0.1)
︡27b6e591-df9b-40cb-937a-6f843650f9f6︡{"stdout": "(array([ 1.71,  1.81,  1.91,  2.01,  2.11,  2.21,  2.31,  2.41,  2.51,\n        2.61,  2.71,  2.81,  2.91]), [0.866020000000000, 1.6347991621959448, 1.6893601550920172, 1.7183588905432456, 1.7466747042757742, 1.7745479694167945, 1.8020015563352152, 1.829053772212458, 1.855721580897002, 1.8820207910761586, 1.9079661643731738, 1.9335715093204211, 1.9588497647662324])"}︡
︠459faf35-099b-4d9e-9a37-fedb2b5a46a5︠
r,u = rk2(0.86602, 1.71, 3, 0.1)
︡0e9fbe7c-9b69-4c3b-bfb3-bd9e93d34bfe︡︡
︠2b03f18f-0266-44c2-b113-632862c6f08f︠
import numpy as np #Transfer files from fundamental library numpy (N-dimensional array manipulation) used with Python.
import matplotlib.pyplot as plt #Transfer files from matplotlib.pyplot (plotting library) used with Python language. 
#Index of Refraction is always greater than 1.  Index of Refraction approaches 1 from above.

x = u
y = r

fig = plt.figure()
ax = fig.add_subplot(111)
ax.plot(x, y)
ax.set_xlim(0.76602, 2.2)
ax.set_ylim(1.6, 3.5)
ax.set_xlabel('Angle of Incidence (u) in radians')
ax.set_ylabel('Radial Distance (r) in (m)*10^4 above mean sea level')
ax.set_title('Numerically Simulated Differential Equations of Refraction')
ax.grid()

plt.savefig('myfig.png')
︡1c725cde-6222-44ef-a622-bfede34fd200︡︡
︠6338fbe1-f0b4-443c-8f21-3d9708eacf03︠
#From the graph above as the measurement along the ray path (s) changes, the radial distance (r) and the angle of incidence (u) changes.
︡e112be54-57b1-41af-9c8c-b7cfae5a9125︡︡
︠decbf5bc-6c83-44d4-8eb5-a0f0bd862624︠
#Use 4th Order Runge-Kutta to integrate equation du/ds = (1-u^2)*(1/r+1/n*dn/dr)
︡daf56c08-56cb-46c3-bb6d-90f0377b0a02︡︡
︠465c485f-adfd-4a1f-8bef-4b50006d67d4︠
import scipy #Transfer files from scipy (open source library of algorithms and mathematical tools from Python programming language) into this worksheet.

def rk4(u0,r0,rn,rt): #4th Order Runge-Kutta defined.

    radial_dis = scipy.arange(r0,rn,rt) #Creates array range (arange) of radial_dis values.
    u=[] #Stores u = dr/ds or derivative of radial distance with respect to (s) measurement along               
         #the ray path.

    for r in radial_dis:
   	u.append(u0) #Attach results (x0) at end of variable x.
   	k1 = udot(r,u0)
   	k2 = udot(r+0.5*rt, u0 + 0.5*rt*k1)
   	k3 = udot(r+0.5*rt, u0 + 0.5*rt*k2)
   	k4 = udot(r+rt, u0 + rt*k3)
   	u0 = u0+(rt/6.0)*(k1 + 2.0*k2 + 2.0*k3 + k4)
    return radial_dis,u
︡c2ddfd9f-1f13-4da1-ac31-03572d3f4748︡︡
︠a335c74e-fed3-42ce-80df-4ae397436a31︠
rk4(0.86602, 1.71, 3, 0.1)
︡c8ace205-65b2-4f83-8d2e-bd889dca3983︡{"stdout": "(array([ 1.71,  1.81,  1.91,  2.01,  2.11,  2.21,  2.31,  2.41,  2.51,\n        2.61,  2.71,  2.81,  2.91]), [0.866020000000000, 0.97460443773712235, 1.0719728174147416, 1.1593986649779751, 1.2381088562350016, 1.3092357190759094, 1.3737956206639728, 1.4326839736496095, 1.486679711579751, 1.5364546840656059, 1.5825851336925272, 1.625563573562677, 1.6658101345483927])"}︡
︠e30b0628-fae4-4311-8e94-219f1e42d2b7︠
r1,u1 = rk4(0.86602, 1.71, 3, 0.1)
︡e72d2bad-7051-45b9-8a67-c32029c9e70a︡︡
︠4a2851a9-5fdd-4eec-b471-9c17304f0b33︠
import numpy as np #Transfer files from fundamental library numpy (N-dimensional array manipulation) used with Python.
import matplotlib.pyplot as plt #Transfer files from matplotlib.pyplot (plotting library) used with Python language. 
#Index of Refraction is always greater than 1.  Index of Refraction approaches 1 from above.

x = u1
y = r1

fig = plt.figure()
ax = fig.add_subplot(111)
ax.plot(x, y)
ax.set_xlim(0.76602, 1.8)
ax.set_ylim(1.6, 3.5)
ax.set_xlabel('Angle of Incidence (u) in radians')
ax.set_ylabel('Radial Distance (r) in (m)*10^4 above mean sea level')
ax.set_title('Numerically Simulated Differential Equations of Refraction')
ax.grid()

plt.savefig('myfig.png')
︡ebbe3fa1-0e84-4a7a-92f8-64d8370fdb4a︡︡
